
local util = {}

local lfs = require("lfs")

function util.write_file(name, data)
   local fd = io.open(name, "w")
   if not fd then
      return nil, "could not write "..name
   end
   fd:write(data)
   fd:close()
   return true
end

function util.read_file(name, offset)
   offset = offset or 0
   local fd = io.open(name, "r")
   if not fd then
      return nil, "could not read "..name
   end
   local n_bytes = fd:seek("set", offset)
   if not n_bytes then
      return nil, "could not read "..name
   end
   local data = fd:read("*a")
   fd:close()
   return data
end

function util.debug_table(tbl, lvl)
   lvl = lvl or 0
   local indent = ("   "):rep(lvl)
   local out = {"{\n"}
   for k,v in pairs(tbl) do
      table.insert(out, indent.."   ")
      table.insert(out, tostring(k).." = ")
      table.insert(out, (type(v) == "table" and util.debug_table(v, lvl + 1) or tostring(v)))
      table.insert(out, "\n")
   end
   table.insert(out, indent.."}")
   return table.concat(out)
end

local function can_open(path, mode)
   local fd = io.open(path, mode)
   return fd and fd:close() or false
end

function util.check_local_path(path_name)
   local attrs = lfs.attributes(path_name)
   if attrs then
      return {
         path = path_name,
         size_bytes = attrs.size,
         is_dir = attrs.mode == "directory",
         is_symlink = attrs.mode == "link",
         is_readable = can_open(path_name, "r"),
         is_writable = can_open(path_name, "w"),
         is_executable = true, --FIXME not obtainable using lfs
         link_path = path_name, --FIXME not obtainable using lfs
      }
   end
   return nil, "Could not read path"
end

function util.convert_to_list(map, key)
   local i = 1
   local tab = {}
   while (map[key.."."..i]) do
      table.insert(tab, map[key.."."..i])
      i = i + 1
  end
  return tab
end

return util
